#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
import re
from pyutilb.cmd import *
from pyutilb.file import *
from pyutilb.ts import age2seconds
from pyutilb.util import replace_sysarg

# 修正k8s命令输出
def fix_k8s_cmd_output(cmd):
    # get pod命令输出的RESTARTS(重启次数)列有括号，如1 (9h ago) => 去掉
    def fix(output):
        if ' get pod' in cmd:
            return re.sub(r"\(.+\)", "", output)
        return output
    return fix

# 同步执行命令，并将输出整理为df + 修正k8s命令输出
def run_command_return_dataframe2(cmd):
    return run_command_return_dataframe(cmd, fix_k8s_cmd_output(cmd))

# --------------------------- k8s命令帮助方法 ---------------------------
# 配置文件
# config_file = '~/.kube/k8scmd.yml'
config_file = os.environ['HOME'] + '/.kube/k8scmd.yml'
# 默认配置
default_config = {
    'get-output': 'wide', # 输出格式
    'get-labels': False, # 是否显示标签
    'get-ns': '', # 命名空间，默认显示全部
}

# 读配置
def read_config():
    if not os.path.exists(config_file):
        write_file(config_file, yaml.dump(default_config))
    return read_yaml(config_file)

# 写配置
def write_config(config):
    #config = dict(default_config, **config)
    # config = {**default_config, **config}
    write_file(config_file, yaml.dump(config))

# 执行命令
def run_cmd(cmd):
    cmd = replace_sysarg(cmd)
    print(cmd)
    os.system(cmd)

# 执行k8s资源的get/describe/edit/delete增删改查命令
def run_res_crud_cmd(res):
    cmd = get_res_crud_cmd(res)
    run_cmd(cmd)

# 生成k8s资源的get/describe/edit/delete增删改查命令
def get_res_crud_cmd(res):
    name = get_res_name(res, False)
    # 1 delete
    if has_delete_arg():
        return f'kubectl delete {res} {name} $2_'

    # 2 edit
    if has_edit_arg():
        return f'kubectl edit {res} {name} $2_'

    # 3 有资源名或label(以-l开头): describe 详情
    if name is not None and not name.startswith('-l '):
        if '-o' in sys.argv: # 有指定输出就用get
            return f'kubectl get {res} {name} $2_'
        # 否则用 describe
        return f'kubectl describe {res} {name} $2_'

    # 4 无资源名: get 列表
    option = build_list_option()

    # 4.3 过滤标签
    labels = '' # 标签
    other_args = '$1_' # 其他参数
    if name is not None and name.startswith('-l '):
        labels = ' ' + name
        other_args = '$2_'

    # 拼接命令
    return f'kubectl get {res}{labels} {option} {other_args}'

# 根据配置构建list显示选项
def build_list_option():
    config = read_config()
    # option = '-o wide --get-labels'
    # 4.1 过滤命名空间
    option = build_ns_option(True)
    # 4.2 输出格式
    if '-o' not in sys.argv:
        option += f" -o {config['get-output']}"
    # 4.3 显示标签
    if config['get-labels']:
        option += ' --show-labels'
    return option

def build_ns_option(is_list):
    '''
    根据配置构建命名空间选项
    :param: is_list 是否是list命令
    '''
    config = read_config()
    option = ''
    if '-n' not in sys.argv and '-A' not in sys.argv:
        if config['get-ns']:
            option = f"-n {config['get-ns']}"
        elif is_list and not sys.argv[0].endswith('cwft'):
            option = '-A'
    return option

# 从命令行参数选出并删掉 -d
def has_delete_arg():
    return has_and_remove_arg('-d')

# 从命令行参数选出并删掉 -e
def has_edit_arg():
    return has_and_remove_arg('-e')

# 从命令行参数选出并删掉指定参数
def has_and_remove_arg(arg):
    ret = arg in sys.argv
    if ret:  # 删除
        sys.argv.remove(arg)
    return ret

res_without_ns = ['no' , 'ns' , 'cs' , 'pv'] # 不带命名空间的资源

# 从命令行参数中资源名或label
def get_res_name(res, required = True):
    if len(sys.argv) == 1:  # 无资源名参数
        if required:
            name = get_latest_res_name(res)
            if name is None:
                raise Exception('缺少资源名参数')
            return name
        return None

    # 取第一个参数为资源名
    name = sys.argv[1]
    # get命令选项，如 -o yaml，是拿不到资源名的
    if name.startswith('-'):
        return None

    # 最新的资源
    if name == '@latest':
        return get_latest_res_name(res)

    # label，其中 @ 是 app= 的缩写
    if name.startswith('@'):
        return f"-l app={name[1:]}"

    # 纯资源名，要带命名空间
    if res == 'pod' and ':' in name: # podname:container
        name, container = name.split(':', 1)
    else:
        container = None
    if '*' in name: # 模糊搜索资源名
        name = search_res_name(res, name)
    elif res not in res_without_ns: # 精确资源名, 找到命名空间 -- 只对带命名空间的资源
        name = name + ' -n ' + get_res_ns(res, name)
    if container:
        name += ' -c ' + container
    return name

# 找到某个资源的命名空间
def get_res_ns(res, name):
    df = run_command_return_dataframe2(f"kubectl get {res} -A -o wide")
    name2ns = dict(zip(df['NAME'], df['NAMESPACE']))
    if name in name2ns:
        return name2ns[name]
    raise Exception(f'找不到{res}资源[{name}]的命名空间')

# 模糊搜索某个资源
def search_res_name(res, name):
    reg = name.replace('*', '.*')
    df = run_command_return_dataframe2(f"kubectl get {res} -A -o wide")
    # 如果是pod，则要对status排序，将Running状态提上去 => 优先取Running状态的pod
    if res == 'pod':
        df['sort'] = list(map(is_pod_running, df['STATUS']))
        df = df.sort_values(by='sort', ascending=False)
    # 匹配资源名
    for i, row in df.iterrows():
        if re.match(reg, row['NAME']):
            if res not in res_without_ns: # 只对带命名空间的资源
                return row['NAME'] + ' -n ' + row['NAMESPACE']
            return row['NAME']
    raise Exception(f'模糊搜索不到{res}资源[{name}]')

# 找到最新的资源名: 按age升序第一个为最新
def get_latest_res_name(res):
    df = run_command_return_dataframe2(f"kubectl get {res} -A -o wide")
    if len(df) == 0:
        return None
    # 对age排序
    df['sort'] = list(map(age2seconds, df['AGE']))
    df = df.sort_values(by='sort', ascending=True)
    row = df.iloc[0]
    return row['NAME'] + ' -n ' + row['NAMESPACE']

def is_pod_running(status):
    return int(status == 'Running')

# ip对pod名的映射
ip2pod = None
# 根据ip找到pod名
def get_pod_by_ip(ip):
    global ip2pod
    if ip2pod is None:
        # 修正RESTARTS列中有`3 (97m ago)`，干掉括号
        def fix_output(o):
            return re.sub(r'\(.+\)', '', o)  # 干掉括号
        df = run_command_return_dataframe2(f"kubectl get pod -A -o wide", fix_output)
        ip2pod = dict(zip(df['IP'], df['NAME']))
    if ip in ip2pod:
        return ip2pod[ip]
    return None

# --------------------------- argo命令帮助方法 ---------------------------
def run_argo_crud_cmd(type):
    '''
    执行argo的list/get/delete增删改查命令
    :param type: 类型，如空或wf表示流程, cwf表示定时流程, wftmpl表示流程模板
    :return:
    '''
    cmd = get_argo_crud_cmd(type)
    run_cmd(cmd)

def get_argo_crud_cmd(type):
    '''
    生成argo的list/get/delete增删改查命令
    :param type: 类型，如空或wf表示流程, cwf表示定时流程, wftmpl表示流程模板
    :return:
    '''
    cmd_pref = get_argo_cmd_pref(type)
    # list选项
    list_option = build_list_option()
    if len(sys.argv) == 1:  # 无流程名
        return f'{cmd_pref} list {list_option}'

    #name = sys.argv[1]
    name = get_argo_name(type, False)
    if name is None:
        return f'{cmd_pref} list {list_option} $1_'

    # 有label(以-l开头): list
    if name.startswith('-l '):
        return f'{cmd_pref} list -A {name} $2_'

    # 1 delete
    if has_delete_arg():
        return f'{cmd_pref} delete {name} $2_'

    # 2 有资源名: get 详情
    return f'{cmd_pref} get {name} $2_'

def get_argo_cmd_pref(type):
    '''
    获得argo命令前缀
    :param type: 类型，如空或wf表示流程, cwf表示定时流程, wftmpl/wft表示流程模板, cwft表示集群级流程模板
    :return:
    '''
    if type == 'cwf':
        cmd_pref = 'argo cron'
    elif type == 'wftmpl' or type == 'wft':
        cmd_pref = 'argo template'
    elif type == 'cwft':
        cmd_pref = 'argo cluster-template'
    else:
        cmd_pref = 'argo'
    return cmd_pref

# 从命令行参数中流程名
def get_argo_name(type, required = True):
    if len(sys.argv) == 1:  # 无流程名参数, 默认给 @latest(最新的流程)
        if required:
            return '@latest ' + build_ns_option(False)
        return None

    # 取第一个参数为流程名
    name = sys.argv[1]
    # get命令选项，如 -o yaml，是拿不到流程名的
    if name.startswith('-'):
        return None

    # label，其中 @ 是 app= 的缩写
    if name.startswith('@') and name != '@latest':
        return f"-l flow={name[1:]}"

    if ':' in name: # flowname:container
        name, container = name.split(':', 1)
    else:
        container = None
    # 纯流程名，要带命名空间
    if '*' in name: # 模糊搜索流程名
        name = search_argo_name(type, name)
    elif '@latest' != name: # 精确流程名, 找到命名空间
        name = name + ' -n ' + get_argo_ns(type, name)
    if container:
        name += ' -c ' + container
    return name

# 模糊搜索某个流程
def search_argo_name(type, name):
    cmd_pref = get_argo_cmd_pref(type)
    reg = name.replace('*', '.*')
    df = run_command_return_dataframe2(f"{cmd_pref} list -A")
    # 匹配流程名
    ret = []
    for i, row in df.iterrows():
        if re.match(reg, row['NAME']):
            name = row['NAME'] + ' -n ' + row['NAMESPACE']
            ret.append(name)
    n = len(ret)
    if n == 0:
        raise Exception(f'模糊搜索不到流程[{name}]')
    if n == 1:
        return ret[0]
    lines = [f"{i+1}. {v}" for i, v in enumerate(ret)]
    i = input("找到多个流程: \n" + "\n".join(lines) + "\n请输入序号选择以上一个流程: ")
    i = int(i) - 1
    if i < 0 or i >= n:
        raise Exception("无效流程序号")
    return ret[i]

# 找到某个流程的命名空间
def get_argo_ns(type, name):
    cmd_pref = get_argo_cmd_pref(type)
    df = run_command_return_dataframe2(f"{cmd_pref} list -A")
    name2ns = dict(zip(df['NAME'], df['NAMESPACE']))
    if name in name2ns:
        return name2ns[name]
    raise Exception(f'找不到流程[{name}]的命名空间')

if __name__ == '__main__':
    print(get_res_name("pod", required=True))


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import sys
from pyutilb.strs import substr_after_last
from k8scmd.util import *

# --------------------------- k8s命令 ---------------------------
'''
k8s命令简写
命名参考 k8s资源简写: https://zhuanlan.zhihu.com/p/369647740
'''
def k8sstart():
    cmd = '''sudo swapoff -a
sudo systemctl restart kubelet
kubectl get nodes'''
    run_cmd(cmd)

def k8sstop():
    cmd = 'sudo systemctl stop kubelet'
    run_cmd(cmd)

def k8scluster():
    # 集群状态
    # run_res_cmd('cs')
    cmd = get_res_crud_cmd('cs')
    if ' get ' in cmd and ' -o wide' in cmd:
        run_cmd('kubectl cluster-info') # 先输出集群信息
        print("\n--------------------------------\n")
    run_cmd(cmd)

# 查看所有资源
def k8sall():
    option = build_list_option()
    run_cmd(f'kubectl get all {option}')

def k8sns():
    run_res_crud_cmd('ns')

def k8sno():
    run_res_crud_cmd('no')

def k8spod():
    run_res_crud_cmd('pod')

def k8ssvc():
    run_res_crud_cmd('svc')

def k8swf():
    run_res_crud_cmd('wf')

def k8swft():
    run_res_crud_cmd('wftmpl')

def k8scwft():
    run_res_crud_cmd('cwft')

def k8scwf():
    run_res_crud_cmd('cwf')

def k8swfevent():
    run_res_crud_cmd('WorkflowEventBinding')

# 输出每个服务的服务url+终端url
def k8ssvcurl():
    cmd = get_res_crud_cmd('svc')
    # get转pandas，并构建http url列，方便用户复制url
    if ' get ' in cmd:
        cmd = replace_sysarg(cmd)
        print(cmd)
        df = run_command_return_dataframe2(cmd)
        # 拼接url
        service_urls = [] # 服务url
        endpoint_urls = [] # 终端的url
        for i, row in df.iterrows():
            service_urls.append(build_service_url(row))
            endpoint_urls.append(build_endpoint_url(row))
        df['SVC-URL'] = service_urls
        df['ENDPOINT-URL'] = endpoint_urls
        del df['CLUSTER-IP']
        del df['EXTERNAL-IP']
        del df['PORT(S)']
        del df['AGE']
        del df['SELECTOR']
        print(df)
        return
    run_cmd(cmd)

# 构建服务url
def build_service_url(row):
    ip = row['CLUSTER-IP']
    ports = re.findall(f'[\d:]+(?=\/)', row['PORT(S)'])
    urls = []
    for port in ports:
        if ':' in port:  # ServicePort:NodePort
            port, _ = port.split(':')
        port = int(port)
        if port != 443 and port != 53:
            url = f"{ip}:{port}"
            urls.append(url)
    return ','.join(urls)

# 构建终端url
def build_endpoint_url(row):
    cmd = f"kubectl describe svc {row['NAME']} -n {get_row_ns(row)}"
    output = run_command(cmd)
    mat = re.search(f'Endpoints:\s*(.+)\n', output)
    ret = mat.group(1)
    return ret
    # return add_pod_by_ip(ret)

# 对ip字符串添加pod名
def add_pod_by_ip(endpoints):
    def add_pod(mat):
        ip = mat.group(1)
        pod = get_pod_by_ip(ip)
        if pod is None:
            return ip
        return f"({pod}){ip}"
    return re.sub(r'(\d+.\d+.\d+.\d+):', add_pod, endpoints)  # 对ip字符串添加pod名

def collect_pod_by_ips(endpoints, has_port = True):
    '''
    对ip字符串收集pod名
    :param endpoints: 终端列表，只能包含ip端口逗号，不能包含其他字符
    :param has_port: 是否包含端口
    :return:
    '''
    port_reg = ''
    if has_port:
        port_reg = ':\d+'
    ips = re.findall(rf'(\d+.\d+.\d+.\d+){port_reg}', endpoints)
    ips = set(ips) # 去重
    return ','.join(list(map(get_pod_by_ip, ips)))

# 获得指定行中的命名空间，如果行中没有，则取配置的默认命名空间
def get_row_ns(row):
    if 'NAMESPACE' in row:
        return row['NAMESPACE']
    # 取配置的默认命名空间
    return read_config()['get-ns']

# 对指定服务url执行curl命令
def k8ssvccurl():
    name = get_res_name('svc')
    res = run_command_return_yaml(f"kubectl get svc {name} -o yaml")
    ip = res['spec']['clusterIP']
    # 遍历每个端口来执行curl
    for p in res['spec']['ports']:
        port = p['port']
        if port != 443 and port != 53:
            curl = f"curl {ip}:{port}"
            # run_cmd(curl)
            print(f"执行命令: {curl}, 结果如下")
            os.system(curl)
            print()

# 输出每个服务的终端对应的pod
def k8ssvcpod():
    cmd = get_res_crud_cmd('endpoints')
    # get转pandas，并构建http url列，方便用户复制url
    if ' get ' in cmd:
        cmd = replace_sysarg(cmd)
        print(cmd)
        df = run_command_return_dataframe2(cmd)
        # 拼接url
        # eps = list(map(add_pod_by_ip, df['ENDPOINTS'])) # 终端加pod -- 显示不全
        # df['ENDPOINTS'] = eps
        pods = []
        for i, row in df.iterrows():
            pods.append(collect_pod_by_ips(row['ENDPOINTS']))
        df['POD'] = pods
        del df['AGE']
        print(df)
        return
    if ' describe ' in cmd:
        cmd = replace_sysarg(cmd)
        print(cmd)
        o = run_command(cmd)
        def add_pods(mat):
            ips = mat.group(2)
            pods = collect_pod_by_ips(ips, False)
            pref = mat.group(1)
            return f"{pref}{ips}\n{pref.replace('Addresses', 'Pods')}{pods}\n"
        o = re.sub(r'( *Addresses: *)((\d+.\d+.\d+.\d+.?)+)\n', add_pods, o)  # 对ip字符串添加pod名
        print(o)
        return
    run_cmd(cmd)

def k8src():
    run_res_crud_cmd('rc')

def k8srs():
    run_res_crud_cmd('rs')

def k8sds():
    run_res_and_rollout_cmd('ds')

def k8ssts():
    run_res_and_rollout_cmd('sts')

def k8sdeploy():
    run_res_and_rollout_cmd('deploy')

# 执行资源+rollout命令(暂停或恢复部署)，用在 deploy + ds + sts
def run_res_and_rollout_cmd(res):
    # 1 执行rollout命令: 暂停或恢复部署，应用在 deploy + ds + sts
    action = get_rollout_action() # 获得rollout命令的动作
    if action:
        name = get_res_name(res)
        cmd = f'kubectl rollout {action} {res} {name}'
        run_cmd(cmd)
        return

    # 2 查看部署：kubectl get/describe deploy
    # run_res_cmd(res)
    cmd = get_res_crud_cmd(res)
    run_cmd(cmd)

    # 3 查看部署状态： kubectl rollout status deploy
    if ' describe ' in cmd:
        print("\n--------------------------------\n")
        name = get_res_name(res)
        cmd = f'kubectl rollout status {res} {name}'
        run_cmd(cmd)

rollout_actions = ['history', 'pause', 'restart', 'resume', 'status', 'undo']

def get_rollout_action():
    '''
    获得rollout命令的动作
        history: 显示上线历史
        pause: 将所指定的资源标记为已暂停
        restart: Restart a resource
        resume: 恢复暂停的资源
        status: 显示上线的状态
        undo: 撤销上一次的上线
    '''
    action = None
    if len(sys.argv) >= 3:
        action = sys.argv[2]
        if action == 'start':  # 恢复部署
            action = 'resume'
        elif action == 'stop':  # 暂停部署
            action = 'pause'
        # restart: 重启pod
    if action in rollout_actions:
        return action
    return None

def k8shpa():
    run_res_crud_cmd('hpa')

def k8sconfig():
    run_res_crud_cmd('cm')

def k8ssecret():
    run_res_crud_cmd('secret')

def k8sendpoint():
    run_res_crud_cmd('endpoints')

def k8sevent():
    run_res_crud_cmd('events')

def k8sjob():
    run_res_crud_cmd('jobs')

def k8sing():
    run_res_crud_cmd('ingresses')

def k8singrule():
    cmd = get_res_crud_cmd('ing')
    # get转pandas，逐个输出rule
    if ' get ' in cmd:
        cmd = replace_sysarg(cmd)
        print(cmd)
        df = run_command_return_dataframe2(cmd)
        for i, row in df.iterrows():
            print('----------------------------')
            print(f"{i+1}. {row['NAME']} -n {get_row_ns(row)}")
            print(build_ingress_rule(row))
        return
    run_cmd(cmd)

# 构建ingress rule
def build_ingress_rule(row):
    cmd = f"kubectl describe ing {row['NAME']} -n {get_row_ns(row)}"
    output = run_command(cmd)
    mat = re.search(f'Rules:\n(.+)\nAnnotations:', output, re.S)
    ret = mat.group(1)
    return ret
    # return add_pod_by_ip(ret)

def k8scron():
    k8scronjob()

def k8scronjob():
    run_res_crud_cmd('cronjobs')

def k8spv():
    run_res_crud_cmd('pv')

def k8spvc():
    run_res_crud_cmd('pvc')

def k8saccount():
    run_res_crud_cmd('serviceaccount')

def k8srole():
    run_res_crud_cmd('role')

def k8srolebind():
    run_res_crud_cmd('rolebinding')

def k8scrole():
    run_res_crud_cmd('clusterrole')

def k8scrolebind():
    run_res_crud_cmd('clusterrolebinding')

# storageclasses
def k8ssc():
    run_res_crud_cmd('storageclasses')

def k8sexec():
    name = get_res_name('pod')
    run_cmd(f'kubectl exec -it {name} -- $2_')

def k8sbash():
    name = get_res_name('pod')
    run_cmd(f'kubectl exec -it {name} -- bash')

def k8ssh():
    name = get_res_name('pod')
    run_cmd(f'kubectl exec -it {name} -- sh')

def k8slog():
    name = get_res_name('pod')
    all_container = ''
    if not ('-c' in sys.argv or ' -c ' in name): # 不带具体容器名
        all_container = '--all-containers=true' # 看所有容器的日志
    run_cmd(f"kubectl logs {name} {all_container} $2_")

def k8sletlog():
    run_cmd("sudo journalctl -u kubelet | tail -n 50")

def build_apply_files():
    # return sys.argv[1] # 可能有多个文件，如对参数 namenode*.yml，命令会拿到多个文件
    files = sys.argv[1:]
    return ' -f '.join(files)

def k8sapply():
    #run_cmd("kubectl apply --record=true -f $1_")
    run_cmd("kubectl apply --record=true -f " + build_apply_files())

def k8screate():
    run_cmd("kubectl create -f " + build_apply_files())

def k8sdelete():
    run_cmd("kubectl delete -f " + build_apply_files())

def k8sdiff():
    run_cmd("kubectl diff -f " + build_apply_files())

def k8sscale():
    # deploy资源名
    name = get_res_name('deploy')
    if len(sys.argv) < 3 or not sys.argv[2].isdigit():
            raise Exception(f'第2个参数必须是int类型的副本数, 而传入的是{sys.argv[2]}')
    # 执行命令
    run_cmd(f"kubectl scale deploy {name} --replicas={sys.argv[2]}")

def k8shistory():
    # deploy资源名
    name = get_res_name('deploy')
    # 命令
    cmd = f"kubectl rollout history deploy {name}"
    # 检查是否有指定版本: 第2个参数
    version = get_deploy_version()
    if version:
        cmd += ' --revision=' + version
    run_cmd(cmd)

def k8srollback():
    # deploy资源名
    name = get_res_name('deploy')
    # 命令
    cmd = f"kubectl rollout undo deploy {name}"
    # 检查是否有指定版本: 第2个参数
    version = get_deploy_version()
    if version:
        cmd += ' --to-revision=' + version
    run_cmd(cmd)

# 获得deploy的版本参数
def get_deploy_version():
    version = None
    if len(sys.argv) >= 3:
        version = sys.argv[2]
        if not version.isdigit():
            raise Exception(f'第2个参数必须是int类型的版本号, 而传入的是{version}')
    return version

# 切换是否显示标签
def k8sgetlabels():
    config = read_config()
    config['get-labels'] = not config['get-labels']
    write_config(config)
    print(f"Set get-labels={config['get-labels']}")

# 指定输出格式
def k8sgetoutput():
    config = read_config()
    if len(sys.argv) == 1: # 无指定格式，则在wide,yaml之间切换
        if config['get-output'] == 'yaml':
            of = 'wide'
        else:
            of = 'yaml'
    else:
        of = sys.argv[1]
    config['get-output'] = of
    write_config(config)
    print(f"Set get-output={config['get-output']}")

# 指定查看的命名空间：只查看该命名空间的资源
def k8sgetns():
    config = read_config()
    if len(sys.argv) == 1:  # 无指定命名空间，则全部
        of = ''
    else:
        of = sys.argv[1]
    config['get-ns'] = of
    write_config(config)
    print(f"Set get-ns={config['get-ns']}")

def k8sapi():
    if len(sys.argv) == 1: # 无资源类型
        run_cmd(f"kubectl api-resources")
        return

    # 有资源类型
    run_cmd(f"kubectl explain {sys.argv[1]} --recursive")

def k8sbuild():
    '''
    使用docker构建镜像，并导入到k8s中
    docker build -t my-app .
    docker save my-app -o my-app.tar
    ctr -n k8s.io images import my-app.tar'''
    # 打包的镜像名
    img = file = ''
    for i in range(0, len(sys.argv)):
        if sys.argv[i] == '-t':
            img = file = sys.argv[i+1]
            if '/' in file: # 192.168.0.182:5000/rpcserver:3.0.0
                file = substr_after_last(file, '/').replace(':', '-')
    if not img:
        raise Exception('没指定镜像名, 请参考docker build -h')
    # 执行命令
    cmd = f'''sudo docker build $1_
    sudo docker save {img} -o {file}.tar
    sudo ctr -n k8s.io images import {file}.tar'''
    run_cmd(cmd)

def k8sexport():
    '''
    k8s导出镜像
    ctr -n k8s.io images export mysql.tar docker.io/library/mysql:5.7-debian --platform linux/amd64
    '''
    img = file = sys.argv[1]
    if '/' in file: # 192.168.0.182:5000/rpcserver:3.0.0
        file = substr_after_last(file, '/').replace(':', '-')
    cmd = f"sudo ctr -n k8s.io images export {file}.tar {img} --platform linux/amd64"
    run_cmd(cmd)

def k8simport():
    '''
    导入镜像
    ctr -n k8s.io images import mysql.tar
    '''
    file = sys.argv[1]
    cmd = f"sudo ctr -n k8s.io images import {file}"
    run_cmd(cmd)

# --------------------------- argo命令 ---------------------------
# ------------- 通用命令 -------------
def argo_create(type):
    cmd_pref = get_argo_cmd_pref(type)
    run_cmd(f"{cmd_pref} create {build_ns_option(False)} $1_")

def argo_resume(type):
    name = get_argo_name(type)
    cmd_pref = get_argo_cmd_pref(type)
    run_cmd(f"{cmd_pref} resume {name} $2_")

def argo_suspend(type):
    name = get_argo_name(type)
    cmd_pref = get_argo_cmd_pref(type)
    run_cmd(f"{cmd_pref} suspend {name} $2_")

# ------------- 流程相关命令 -------------
def wf():
    run_argo_crud_cmd('wf')

def wfsubmit():
    w = ''
    if '--watch' not in sys.argv:
        w = '--watch'
    run_cmd(f"argo submit {build_ns_option(False)} $1_ {w}")

# 删除所有流程
def wfclear():
    r = input("Are you delete all workflow? (Y/N) ").lower()
    if r == 'y' or r == 'yes':
        run_cmd("argo delete -A")

def wflog():
    name = get_argo_name('wf')
    run_cmd(f"argo logs {name} $2_")

def wfretry():
    name = get_argo_name('wf')
    run_cmd(f"argo retry {name} $2_")

def wfresume():
    argo_resume('wf')

def wfsuspend():
    argo_suspend('wf')

# ------------- 定时流程相关命令 -------------
def cwf():
    run_argo_crud_cmd('cwf')

def cwfcreate():
    argo_create('cwf')

def cwfresume():
    argo_resume('cwf')

def cwfsuspend():
    argo_suspend('cwf')

# ------------- 流程模板相关命令 -------------
def wft():
    run_argo_crud_cmd('wft')

def wftcreate():
    argo_create('wft')

# ------------- 集群级流程模板相关命令 -------------
def cwft():
    run_argo_crud_cmd('cwft')

def cwftcreate():
    argo_create('cwft')

# 测试
if __name__ == '__main__':
    # k8spod()
    # k8sexec()
    # k8sbash()
    # k8ssvc()
    # k8singrule()
    # k8ssvcpod()
    # k8sbuild()
    cwft()


# coding=utf-8
import re
import ast
from setuptools import setup, find_packages
from os.path import dirname, join, abspath
import setuptools
from pyutilb.util import *
from pyutilb.file import *
from pyutilb.cmd import *

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# 读元数据：author/version/description
meta = read_init_file_meta('k8scmd/__init__.py')

# 读依赖
with open('requirements.txt', 'rb') as f:
    text = f.read().decode('utf-8')
    text = text.replace(' ', '') # 去掉空格
    requires = text.split('\n')

def build_entry_points():
    '''
    构建命令入口
       格式为['命令名 = 模块名:函数名']， 如 ['k8scmd=k8scmd.boot:main']
    '''
    funs = load_module_funs('k8scmd/cmd.py')
    # return [f'{name}=k8scmd.cmd:{name}' for name, fun in funs.items() if name.startswith('k8s')]
    ret = []
    for name, fun in funs.items():
        if name.startswith('k8s'): # k8s前缀
            ret.append(f'{name}=k8scmd.cmd:{name}')
            alias = name.replace('k8s', 'k') # 别名简写为k前缀
            ret.append(f'{alias}=k8scmd.cmd:{name}')
        if name.startswith('wf') or name.startswith('cwf'): # wf/wft/cwf前缀
            ret.append(f'{name}=k8scmd.cmd:{name}')
    return ret

setup(
    name='k8scmd',
    version=meta['version'],
    url='https://github.com/shigebeyond/k8scmd',
    license='BSD',
    author=meta['author'],
    author_email='772910474@qq.com',
    description=meta['description'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6",
    install_requires=requires,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Operating System :: POSIX :: Linux",
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    # 把python中的函数自动生成为一个可执行的脚本
    entry_points={
       'console_scripts': build_entry_points()
    },
)



import k8scmd

__author__ = "shigebeyond"
__version__ = "1.0.0"
__description__ = "k8scmd: easy way to execute k8s command"

__all__ = [
    "__author__",
    "__version__",
    "__description__",
    "k8scmd",
]